library(MASS)

#--------------------------------------------------------------------------
# - ALGORITHM 1 FOR THE HEAT SENSOR SYSTEM IN CHAPTER 6.1.
#--------------------------------------------------------------------------

set.seed(123)
it <- 1000000
beta <- 0.02
lambda <- 5.8e-6
tau <- 8760
n <- 24
tfail <- 0
twork <- 0

for(j in 1:it) {
	ind = c(0,0,0,0)
	twork <- twork + tau
	y <- rexp(n,lambda)
	ccf <- rexp(1,lambda*beta)

	for (i in 2:(n-1)) {
		if(y[i-1] < tau && y[i] < tau && y[i+1] < tau) {
			ind[1] <-  (tau - max(c(y[i-1],y[i],y[i+1])))			
		}
	}
	if(y[1] < tau && y[n] < tau && y[n-1] <tau) {
		ind[2] <- (tau - max(c(y[1],y[n],y[n-1])))		
	#	print("indep")
	}
	if(y[1] < tau && y[n] < tau && y[2] <tau){
		ind[3] <- (tau - max(c(y[1],y[n],y[2])))			
	#	print("indep")
	}
	if (ccf < tau) {
		ind[4] <- tau-ccf
	#	print("ccf")
	}	
	if (sum(ind) >0) {
		tfail <- tfail + max(ind)
		twork <- twork - max(ind)
	}	
}	
f1 <- tfail/(tfail+twork)
w1 <- twork/(tfail+twork)
f1

#--------------------------------------------------------------------------
# - THE HEAT SENSOR SYSTEM IN CHAPTER 6.2. (ALGORITHM 2) THE SYSTEM WHICH 
# - DETECTS HIGH TEMPERATURE 
#--------------------------------------------------------------------------
 
set.seed(123)
beta <- 0.02
lambda <- 3*5.8e-6
lamb <- (1-beta)*lambda
cmn <-  beta*3*5.8e-6 
m = 4
n=8
tau <- 8760
tGood <- 0
tBad <- 0
it <- 100000

for(i in 1:it) {
	ctime <- rexp(1,cmn)
	tGood <- tGood + tau
	pl <- c()
	pl[1] <- 1
	j <- 1
	wt <- rexp(n,lamb)
	for (i in 1:n) {
		if(wt[i] < tau) {
			pl[j] <- wt[i]
			j <- j+1
		}
	}
	if (length(pl) <=(n-m)){
		pl <- rep(tau,n)
	}	
	if ((length(pl) > (n-m)) || (ctime < tau)) {
		nyg <- sort(pl)
		tBad <- tBad + max((tau-nyg[n-m+1]),(tau-ctime), na.rm=TRUE)
		tGood <- tGood - max((tau-nyg[n-m+1]),(tau-ctime), na.rm=TRUE)
	}
}
f2 <- tBad/(tBad+tGood)
f2

#--------------------------------------------------------------------------
SOME ADDITIONAL CALCULATIONS AND ALGORITHMS
#--------------------------------------------------------------------------

#Exact for 4oo8 when using the beta-factor

PfDx = factorial(n)/(factorial(n-m+1)*factorial(m-1))*((lamb*tau)^(n-m+1))/(n-m+2)
PfDx + beta*lambd*tau/2


#----------------------------------------------------------
# - GENERAL PDS METHOD. THE PARAMETERS n,m beta2,theta,lambda,tau,beta
# - MAY BE CHANGED TO ACCOUNT FOR DIFFERENT SYSTEMS 
#--------------------------------------------------------- 

n=24
m=20
beta2 = 0.3
theta= 0.5
lambda <- 5.8e-6
tau <- 8760
beta <- 0.02

#--CCF-contribution
C <- c()
i <- 1
if ((n-m) > 1) {
	for(j in (n-m+1): n) {
		#print(j)
		C[i] <- beta2*factorial(n)/factorial(j)/factorial(n-j)*(theta^(j-3))*((1-theta)^(n-j))
		i=i+1
	}
}
if((n-m) ==1) {
	C[i] <- factorial(n)/factorial(2)/factorial(n-2) *(1-beta2/theta)
	i <- i+1
	for(j in 2:n) {
		#print(j)
		C[i] <- beta2*factorial(n)/factorial(j)/factorial(n-j)*(theta^(j-3))*((1-theta)^(n-j))
		i=i+1
	}
}
Cmoon <- sum(C)
Cmoon
pfdccf <- Cmoon*beta*lambda*tau/2

#---Independent failure  contribution

Ctot <- c()
y <- 1
for(r in 1:(n-1)){
	i <- 1
	Cn <- c()
	if ((n-r) > 1) {
		for(j in (n-r+1): n) {
			Cn[i] <- beta2*factorial(n)/factorial(j)/factorial(n-j)*(theta^(j-3))*((1-theta)^(n-j))
			i=i+1
		}
	}
	if((n-r) ==1) {
		Cn[i] <- factorial(n)/factorial(2)/factorial(n-2) *(1-beta2/theta)
		i <- i+1
		for(j in 2:n) {
			Cn[i] <- beta2*factorial(n)/factorial(j)/factorial(n-j)*(theta^(j-3))*((1-theta)^(n-j))
			i=i+1
		}
		
	}
	Ctot[y] <- sum(Cn)
	y <- y+1

}
CNm1N <- Ctot[length(Ctot)]
CN <- sum(Ctot)	
Hn <- (CN+CNm1N)/n
Hn
pfdindep <- factorial(n)/factorial(n-m+2)/factorial(m-1)*((1-Hn*beta)*lambda*tau)^(n-m+1) 

pfdccf
pfdindep

pfdccf + pfdindep
