library(MASS)
library(Malmig)
beta <- 0.05
lambda <- 1e-6
tau <- 8760
n <- tau

#---------------------------------------------------
# - EXAMPLE CHAPTER. 3.8.1 
# - DISKRET TIME. THE MATRIX MULTIPLICATION METHOD
#---------------------------------------------------

P0 <- matrix(c(1,0,0),nrow=1,ncol=3)
T <- matrix(c(1-(2-beta)*lambda, 2*(1-beta)*lambda, beta*lambda, 
	0,1-lambda,lambda,
	0,0,1),byrow=TRUE,nrow=3,ncol=3) 
Tx <- c()

for (i in 1:n) {
	Tn <- mtx.exp(T,i)
	Tx[i] <- (P0%*%Tn)[3]
}
plot(Tx,type="l",xlab="Hours",ylab="PFD")

PFDav <- 1/n*sum(Tx)
PFDav

# - EXAMPLE CHAPTER 3.8.2.
# - CONTINUOUS TIME. THE DIFFERENTIAL EQUATION METHOD

t <- 1:tau
P3 <- 1- 2*exp(-lambda*t) + exp(-(2-beta)*lambda*t)
PFDavanalytic <- 1/n*sum(P3)
PFDavanalytic

# - PLOT IN FIGURE 3.7. 
# - THE DIFFERENCE BETWEEN THE RESULT IN 3.8.1 AND 3.8.2

plot(P3-Tx,type="l",cex.axis = 1.5,cex.lab=1.7,xlab="Hours",ylab="Difference")


#---------------------------------------------------
# - THE OIL-SYSTEM IN CHAPTER 4.4. 
# - THE PFD CALCULATIONS WHEN APPLYING THE MATRIX MULTIPLICATION METHOD.
#---------------------------------------------------
lis <- 2.38e-7
l34 <- 3.4e-5
beta <- 0.02
tau <- 8760
n <- tau

P0o <- matrix(c(1,0,0,0,0,0),nrow=1,ncol=6)
To <- matrix(c(1-((2-beta)*l34 +lis), 2*l34*(1-beta), lis, beta*l34,0,0
      ,0, 1-(lis+l34),0,l34,lis,0
	,0,0, 1-(2-beta)*l34, 0, 2*l34*(1-beta), beta*l34
	,0, 0, 0, 1-lis, 0, lis
	,0, 0, 0, 0, 1-l34, l34
	,0,0,0,0,0,1),byrow=TRUE,nrow=6,ncol=6)

Tx <- c()
for(i in 1:n) {
	Tn0 <- mtx.exp(To,i)
	Tx[i] <- sum((P0o%*%Tn0)[4:6])
}
plot(Tx,type="l",cex.axis = 1.5,cex.lab=1.7,xlab="Hours",ylab="PFD")

PFDavoil <- 1/n*sum(Tx)
PFDavoil	
#---------------------------------------------------
# - THE SPEED-SENSOR SYSTEM IN CHAPTER 5.3.
# - THE PFD CALCULATIONS WHEN APPLYING THE MATRIX MULTIPLICATION METHOD.
#---------------------------------------------------
beta   <- 0.02
lambda <- 2.3e-6
tau <- 8760
li <- (1-beta) * lambda
lc <- beta * lambda

y <- matrix(0,nrow=8,ncol=8)

y[,1] <- c(1-(6-5*beta)*lambda, 0, 0, 0, 0, 0, 0, 0)
y[,2] <- c(6*li, 1-(5-4*beta)*lambda, 0, 0, 0, 0, 0, 0)
y[,3] <- c(1/5*lc, 5*li, 1-(4-3*beta)*lambda, 0, 0, 0, 0, 0)
y[,4] <- c(1/5*lc, 1/4*lc, 4*li,1-(3-2*beta)*lambda, 0, 0, 0, 0)
y[,5] <- c(6/15/5 *lc, 6/15/4 *lc, 6/15/3 *lc, 6/15*3*li, 1-(2-beta)*lambda,0,0,0)
y[,6] <- c(9/15/5*lc,9/15/4*lc,9/15/3*lc,9/15*3*li,0,1-(2-beta)*lambda,0,0)
y[,7] <- c(1/5*lc,1/4*lc,1/3*lc,1/2*lc,2*li,2*li,1-lambda,0)
y[,8] <- c(1/5*lc,1/4*lc,1/3*lc,1/2*lc,lc,lc,lambda,1)

p0 <- matrix(c(1,0,0,0,0,0,0,0),nrow=1,ncol=8)


Tx <- c()
for(i in 1:n) {
	Tn0 <- mtx.exp(y,i)
	Tx[i] <- sum((p0%*%Tn0)[6:8])
}
plot(Tx,type="l",cex.axis = 1.5,cex.lab=1.7,xlab="Hours",ylab="PFD")

p0%*%mtx.exp(y,tau)

PFDavoil <- 1/n*sum(Tx)
PFDavoil	

#---------------------------------------------------
# - THE HEAT SENSOR SYSTEM IN CHAPTER 6.2.3
# - THE PFD CALCULATIONS WHEN APPLYING THE MATRIX MULTIPLICATION METHOD.
#---------------------------------------------------

beta= 0.02
lambda = 1.74e-5
tau = 8760
n <- tau
li <- (1-beta)*lambda
lc <- beta*lambda

H <- matrix(0,nrow=6,ncol=6)
H[,1] <- c(1-(8-7*beta)*lambda,0,0,0,0,0)
H[,2] <- c(8*li,1-(7-6*beta)*lambda,0,0,0,0)
H[,3] <- c(lc/7, 7*li,1-(6-5*beta)*lambda,0,0,0)
H[,4] <- c(lc/7, lc/6, 6*li, 1-(5-4*beta)*lambda,0,0)
H[,5] <- c(lc/7,lc/6,lc/5, 5*li,1-4*lambda,0)
H[,6] <- c(4*lc/7,4*lc/6,4*lc/5,lc,4*lambda,1)


p0 <- matrix(c(1,0,0,0,0,0),nrow=1,ncol=6)


Tx <- c()
for(i in 1:n) {
	Tn0 <- mtx.exp(H,i)
	Tx[i] <- (p0%*%Tn0)[6]
}
plot(Tx,type="l",cex.axis = 1.5,cex.lab=1.7,xlab="Hours",ylab="PFD")

p0%*%mtx.exp(H,tau)

PFDavoil <- 1/n*sum(Tx)
PFDavoil	










